@@ -2,7 +2,7 @@
 
 ################################################################################
 ###                                                                          ###
-### Created by Martin Genet, 2016-2020                                       ###
+### Created by Martin Genet, 2016-2021                                       ###
 ###                                                                          ###
 ### École Polytechnique, Palaiseau, France                                   ###
 ###                                                                          ###
@@ -50,18 +50,3 @@ class Energy():
             **kwargs):
 
         pass
-
-
-
-    def get_qoi_names(self):
-
-        return [self.name+"_ener"]
-
-
-
-    def get_qoi_values(self):
-
-        self.ener = (dolfin.assemble(self.ener_form)/self.problem.mesh_V0)**(1./2)
-        self.printer.print_sci(self.name+"_ener",self.ener)
-
-        return [self.ener]
