@@ -2,7 +2,7 @@
 
 ################################################################################
 ###                                                                          ###
-### Created by Martin Genet, 2016-2020                                       ###
+### Created by Martin Genet, 2016-2021                                       ###
 ###                                                                          ###
 ### École Polytechnique, Palaiseau, France                                   ###
 ###                                                                          ###
@@ -34,6 +34,8 @@ def compute_strains(
         disp_array_name="displacement",
         defo_grad_array_name="DeformationGradient",
         strain_array_name="Strain",
+        jacobian_array_name="Jacobian",
+        equiv_dev_strain_array_basename="EquivDeviatoric",
         ref_mesh_folder=None,                       # MG20190612: Mesh with sectors/parts/etc.
         ref_mesh_basename=None,
         ref_mesh_ext="vtk",
@@ -186,6 +188,18 @@ def compute_strains(
             strain_array_name=strain_array_name,
             mesh_w_local_basis=ref_mesh,
             verbose=verbose)
+        ### 20210601 Modif CL
+        myvtk.addJacobiansFromDeformationGradients(
+                mesh=mesh,
+                defo_grad_array_name=defo_grad_array_name,
+                jacobian_array_name=jacobian_array_name,
+                verbose=verbose)
+        myvtk.addEquivDeviatoricStrainsFromDeformationGradients(
+                mesh=mesh,
+                defo_grad_array_name=defo_grad_array_name,
+                equiv_dev_strain_array_basename=equiv_dev_strain_array_basename,
+                verbose=verbose)
+        ###
         if (ref_mesh is not None):
             if  (iarray_ref_part_id is not None)\
             and (remove_boundary_layer         ):
