@@ -2,7 +2,7 @@
 
 ################################################################################
 ###                                                                          ###
-### Created by Martin Genet, 2016-2020                                       ###
+### Created by Martin Genet, 2016-2021                                       ###
 ###                                                                          ###
 ### École Polytechnique, Palaiseau, France                                   ###
 ###                                                                          ###
@@ -57,7 +57,6 @@ def get_ExprGenIm_cpp_pybind(
 #include <vtkCellData.h>
 #include <vtkDataArray.h>
 #include <vtkDoubleArray.h>
-#include <vtkDoubleArray.h>
 #include <vtkImageData.h>
 #include <vtkImageGradient.h>
 #include <vtkImageInterpolator.h>
@@ -72,6 +71,11 @@ def get_ExprGenIm_cpp_pybind(
 #include <vtkXMLImageDataWriter.h>
 #include <vtkXMLUnstructuredGridWriter.h>
 
+#include <vtkImageFFT.h>
+#include <vtkImageRFFT.h>
+#include <vtkImageExtractComponents.h>
+#include <vtkImageMathematics.h>
+
 #include <pybind11/eigen.h>
 #include <pybind11/pybind11.h>
 
@@ -87,6 +91,7 @@ public:
 
     vtkSmartPointer<vtkXMLImageDataReader> reader;
     vtkSmartPointer<vtkImageData>          image;
+    vtkSmartPointer<vtkImageData>          image_output;
     vtkSmartPointer<vtkImageData>          image_up;'''+('''
     vtkSmartPointer<vtkImageGradient>      grad;
     vtkSmartPointer<vtkImageData>          grad_image;''')*(im_type=="grad")+'''
@@ -97,6 +102,12 @@ public:
     vtkSmartPointer<vtkProbeFilter>        probe;
     vtkSmartPointer<vtkImageData>          probe_image;
 
+    vtkSmartPointer<vtkImageFFT>               fft;
+    vtkSmartPointer<vtkImageData>              image_downsampled;
+    vtkSmartPointer<vtkDoubleArray>            image_downsampled_scalars;
+    vtkSmartPointer<vtkImageRFFT>              rfft;
+    vtkSmartPointer<vtkImageExtractComponents> extract;
+
     std::shared_ptr<dolfin::Mesh>     mesh;
     std::shared_ptr<dolfin::Function> U;
 
@@ -113,7 +124,12 @@ public:
             interpolator(vtkSmartPointer<vtkImageInterpolator>::New()),
             ugrid(vtkSmartPointer<vtkUnstructuredGrid>::New()),
             warp(vtkSmartPointer<vtkWarpVector>::New()),
-            probe(vtkSmartPointer<vtkProbeFilter>::New())
+            probe(vtkSmartPointer<vtkProbeFilter>::New()),
+            fft(vtkSmartPointer<vtkImageFFT>::New()),
+            image_downsampled(vtkSmartPointer<vtkImageData>::New()),
+            image_downsampled_scalars(vtkSmartPointer<vtkDoubleArray>::New()),
+            rfft(vtkSmartPointer<vtkImageRFFT>::New()),
+            extract(vtkSmartPointer<vtkImageExtractComponents>::New())
     {'''+('''
         std::cout << "constructor" << std::endl;''')*(verbose)+('''
 
@@ -123,7 +139,7 @@ public:
         reader->UpdateDataObject();
         image = reader->GetOutput();'''+('''
 
-        grad->SetInputDataObject(image_up);
+        grad->SetInputDataObject(image_up); // FA20200217: image_up or image_output?
         grad->SetDimensionality(n_dim);
         grad->UpdateDataObject();
         grad_image = grad->GetOutput();''')*(im_type=="grad")+'''
@@ -190,10 +206,10 @@ public:
         // std::cout << image_up->GetPointData()->GetScalars()->GetNumberOfTuples() << std::endl;
         // std::cout << image_up->GetPointData()->GetScalars()->GetNumberOfComponents() << std::endl;'''+('''
 
-        interpolator->Initialize(image_up);''')*(im_type=="im")+('''
+        //interpolator->Initialize(image); // FA20200217: moved to generate_image, with Initialize(image_output)''')*(im_type=="im")+('''
 
         grad->Update();
-        interpolator->Initialize(grad_image);''')*(im_type=="grad")+'''
+        interpolator->Initialize(grad_image); // FA20200217: also this needs to be moved to generate_image?''')*(im_type=="grad")+'''
     }
 
     void init_ugrid
@@ -369,8 +385,206 @@ public:
         ugrid->Modified();
     }
 
+    void compute_downsampled_image
+    (
+        double          downsampling_factor_input,
+        unsigned int    write_temp_images=0,
+        const char*     suffix = "" // FA20200217: suffix for temp_images
+    )
+    {'''+('''
+        // FA20200214: Function to downsampled image_up, create "extract" object with the downsampled image
+        std::cout << "compute_downsampled_images" << std::endl;''')*(verbose)+'''
+
+        unsigned int image_up_ndim = n_dim;
+        '''+('''std::cout << "image_up_ndim = " << image_up_ndim << std::endl;''')*(verbose)+'''
+
+        int image_up_dimensions[3];
+        image_up->GetDimensions(image_up_dimensions);
+        '''+('''std::cout << "image_up_dimensions = " << image_up_dimensions[0] << " " << image_up_dimensions[1] << " " << image_up_dimensions[2] << std::endl;''')*(verbose)+'''
+
+        double image_up_origin[3];
+        image_up->GetOrigin(image_up_origin);
+        '''+('''std::cout << "image_up_origin = " << image_up_origin[0] << " "<< image_up_origin[1] << " " << image_up_origin[2] << std::endl;''')*(verbose)+'''
+
+        double image_up_spacing[3];
+        image_up->GetSpacing(image_up_spacing);
+        '''+('''std::cout << "image_up_spacing = " << image_up_spacing[0] << " " << image_up_spacing[1] << " " << image_up_spacing[2] << std::endl;''')*(verbose)+('''
+
+        double downsampling_factors[3] = {downsampling_factor_input, downsampling_factor_input, 1};''')*(im_dim==2)+('''
+        double downsampling_factors[3] = {downsampling_factor_input, downsampling_factor_input, downsampling_factor_input};''')*(im_dim==3)+('''
+
+        std::cout << "downsampling_factors = ";
+        std::copy(std::begin(downsampling_factors),std::end(downsampling_factors),std::ostream_iterator<int>(std::cout, "\n"));''')*(verbose)+'''
+
+        double image_up_downsampled_dimensions_db[3];
+        std::transform(image_up_dimensions, image_up_dimensions+3, downsampling_factors, image_up_downsampled_dimensions_db, std::divides<double>());
+        image_up_downsampled_dimensions_db[0] = std::ceil(image_up_downsampled_dimensions_db[0]);
+        image_up_downsampled_dimensions_db[1] = std::ceil(image_up_downsampled_dimensions_db[1]);
+        image_up_downsampled_dimensions_db[2] = std::ceil(image_up_downsampled_dimensions_db[2]);
+        int image_up_downsampled_dimensions[3];
+        image_up_downsampled_dimensions[0] = int (image_up_downsampled_dimensions_db[0]);
+        image_up_downsampled_dimensions[1] = int (image_up_downsampled_dimensions_db[1]);
+        image_up_downsampled_dimensions[2] = int (image_up_downsampled_dimensions_db[2]);
+        '''+('''std::cout << "image_up_downsampled_dimensions = " << image_up_downsampled_dimensions[0] << " " << image_up_downsampled_dimensions[1] << " " << image_up_downsampled_dimensions[2] << std::endl;''')*(verbose)+'''
+
+        fft->SetDimensionality(image_up_ndim);
+        fft->SetInputData(image_up);
+        fft->UpdateDataObject();
+
+        vtkSmartPointer<vtkXMLImageDataWriter> writer_fft;
+        if (write_temp_images)
+        {
+            writer_fft = vtkSmartPointer<vtkXMLImageDataWriter>::New();
+            writer_fft->SetInputData(fft->GetOutput());
+        }
+
+        int image_up_downsampled_npoints = image_up_downsampled_dimensions[0]*image_up_downsampled_dimensions[1]*image_up_downsampled_dimensions[2];
+        '''+('''std::cout << "image_up_downsampled_npoints = " << image_up_downsampled_npoints << std::endl;''')*(verbose)+'''
+
+        std::transform(image_up_dimensions, image_up_dimensions+3, image_up_downsampled_dimensions, downsampling_factors, std::divides<double>());
+        '''+('''std::cout << "downsampling_factors = " << downsampling_factors[0] << " " << downsampling_factors[1] << " " << downsampling_factors[2] << std::endl;''')*(verbose)+'''
+
+        double downsampling_factor = downsampling_factors[0]*downsampling_factors[1]*downsampling_factors[2];
+        '''+('''std::cout << "downsampling_factor = " << downsampling_factor << std::endl;''')*(verbose)+'''
+
+        double image_downsampled_origin[3];
+        std::copy(std::begin(image_up_origin), std::end(image_up_origin), std::begin(image_downsampled_origin));
+        '''+('''std::cout << "image_downsampled_origin = " << image_downsampled_origin[0] << " " << image_downsampled_origin[1] << " " << image_downsampled_origin[2] << std::endl;''')*(verbose)+'''
+
+        double image_downsampled_spacing[3];
+        std::transform(image_up_spacing, image_up_spacing+3, downsampling_factors, image_downsampled_spacing, std::multiplies<double>());
+        '''+('''std::cout << "image_downsampled_spacing = " << image_downsampled_spacing[0] << " " << image_downsampled_spacing[1] << " " << image_downsampled_spacing[2] << std::endl;''')*(verbose)+'''
+
+        image_downsampled->SetDimensions(image_up_downsampled_dimensions);
+        image_downsampled->SetOrigin(image_downsampled_origin);
+        image_downsampled->SetSpacing(image_downsampled_spacing);
+
+        // NOTE: The following corresponds to the function myvtk.createDoubleArray (line 144 python version)
+        int n_components = 2;
+        int n_tuples = image_up_downsampled_npoints;
+        int init_to_zero = 0;
+
+        image_downsampled_scalars->SetName("ImageScalars");
+        image_downsampled_scalars->SetNumberOfComponents(n_components);
+        image_downsampled_scalars->SetNumberOfTuples(n_tuples);
+        if (init_to_zero)
+        {
+            for (int k_component = 0; k_component<n_components; k_component++)
+            {
+                image_downsampled_scalars->FillComponent(k_component, 0.);
+            }
+        }
+
+        image_downsampled->GetPointData()->SetScalars(image_downsampled_scalars);
+
+        vtkSmartPointer<vtkXMLImageDataWriter> writer_sel;
+        if (write_temp_images)
+        {
+            writer_sel = vtkSmartPointer<vtkXMLImageDataWriter>::New();
+            writer_sel->SetInputData(image_downsampled);
+        }
+
+        rfft->SetDimensionality(image_up_ndim);
+        rfft->SetInputData(image_downsampled);
+        rfft->UpdateDataObject();
+
+        extract->SetInputData(rfft->GetOutput());
+        extract->SetComponents(0);
+        extract->UpdateDataObject();
+
+        // vtkSmartPointer<vtkXMLImageDataWriter> writer = vtkSmartPointer<vtkXMLImageDataWriter>::New();
+        // writer->SetInputData(extract->GetOutput());
+
+        fft->Update();
+        if (write_temp_images)
+        {
+            std::string writer_fft_name_str = std::string(suffix)+"_fft.vti";
+            char writer_fft_name[writer_fft_name_str.size()+1];
+
+            writer_fft_name_str.copy(writer_fft_name, writer_fft_name_str.size()+1);
+            writer_fft_name[writer_fft_name_str.size()] = '\0';
+
+            writer_fft->SetFileName(writer_fft_name);
+            writer_fft->Write();
+        }
+
+        vtkSmartPointer<vtkDataArray> image_scalars = fft->GetOutput()->GetPointData()->GetScalars();                 // FA20200217: CHECK: Is it well defined the type of this pointer?
+        vtkSmartPointer<vtkDataArray> image_downsampled_scalars_1 = image_downsampled->GetPointData()->GetScalars();  // FA20200217: CHECK: Is it well defined the type of this pointer?
+        int k_x, k_y, k_z;
+        double I[2];
+        for (int k_z_downsampled = 0; k_z_downsampled<image_up_downsampled_dimensions[2]; k_z_downsampled++)
+        {
+            if (k_z_downsampled <= image_up_downsampled_dimensions[2]/2) // FA20200217: NOTE: This should be equivalent to a//b (because it is dividing int/int)
+            {
+                k_z = k_z_downsampled;
+            }
+            else
+            {
+                k_z = k_z_downsampled+(image_up_dimensions[2]-image_up_downsampled_dimensions[2]);
+            }
+
+            for (int k_y_downsampled = 0; k_y_downsampled<image_up_downsampled_dimensions[1]; k_y_downsampled++)
+            {
+                if (k_y_downsampled <= image_up_downsampled_dimensions[1]/2) // FA20200217: NOTE: This should be equivalent to a//b (because it is dividing int/int)
+                {
+                    k_y = k_y_downsampled;
+                }
+                else
+                {
+                    k_y = k_y_downsampled+(image_up_dimensions[1]-image_up_downsampled_dimensions[1]);
+                }
+
+                for (int k_x_downsampled = 0; k_x_downsampled<image_up_downsampled_dimensions[0]; k_x_downsampled++)
+                {
+                    if (k_x_downsampled <= image_up_downsampled_dimensions[0]/2) // FA20200217: NOTE: This should be equivalent to a//b (because it is dividing int/int)
+                    {
+                        k_x = k_x_downsampled;
+                    }
+                    else
+                    {
+                        k_x = k_x_downsampled+(image_up_dimensions[0]-image_up_downsampled_dimensions[0]);
+                    }
+
+                    int k_point_downsampled = k_z_downsampled*image_up_downsampled_dimensions[1]*image_up_downsampled_dimensions[0] + k_y_downsampled*image_up_downsampled_dimensions[0] + k_x_downsampled;
+                    int k_point             = k_z            *image_up_dimensions[1]            *image_up_dimensions[0]             + k_y            *image_up_dimensions[0]             + k_x;
+                    image_scalars->GetTuple(k_point, I);
+                    I[0] /= downsampling_factor;
+                    I[1] /= downsampling_factor;
+                    image_downsampled_scalars_1->SetTuple(k_point_downsampled, I);
+                }
+            }
+            image_downsampled->Modified();
+
+            if (write_temp_images)
+            {
+                std::string writer_sel_name_str = std::string(suffix)+"_sel.vti";
+                char writer_sel_name[writer_sel_name_str.size()+1];
+
+                writer_sel_name_str.copy(writer_sel_name, writer_sel_name_str.size()+1);
+                writer_sel_name[writer_sel_name_str.size()] = '\0';
+
+                writer_sel->SetFileName(writer_sel_name);
+                writer_sel->Write();
+            }
+        }
+
+        rfft->Update();
+
+        extract->Update();
+
+        // FA20200217: The downsampled image can be saved with write_image()
+        // std::string writer_name_str = std::string(suffix)+".vti";
+        // char writer_name[writer_name_str.size()+1];
+        // writer_name_str.copy(writer_name, writer_name_str.size()+1);
+        // writer_name[writer_name_str.size()] = '\0';
+        // writer->SetFileName(writer_name);
+        // writer->Write();
+
+    }
+
     void generate_image
     (
+        double n_down=1
     )
     {'''+('''
         std::cout << "generate_image" << std::endl;''')*(verbose)+'''
@@ -410,7 +624,14 @@ public:
             }
             image_up_scalars->SetTuple(k_point, I);
         }
-        image_up->Modified();'''+('''
+        image_up->Modified();
+
+        compute_downsampled_image(n_down);
+        image_output = extract->GetOutput(); // FA20200217: this has the information of the downsampled image
+
+        interpolator->Initialize(image_output);
+
+        '''+('''
 
         grad->Update();''')*(im_type=="grad")+'''
     }
@@ -423,7 +644,7 @@ public:
         std::cout << "write_image" << std::endl;''')*(verbose)+'''
 
         vtkSmartPointer<vtkXMLImageDataWriter> writer = vtkSmartPointer<vtkXMLImageDataWriter>::New();
-        writer->SetInputData(image_up);
+        writer->SetInputData(image_output); // FA20200217: changed image_up to image_output
         writer->SetFileName(filename);
         writer->Write();
     }'''+('''
@@ -516,7 +737,8 @@ PYBIND11_MODULE(SIGNATURE, m)
     .def("init_image", &'''+name+'''::init_image, pybind11::arg("filename"), pybind11::arg("n_up") = 1)
     .def("init_ugrid", &'''+name+'''::init_ugrid, pybind11::arg("mesh_"), pybind11::arg("U_"))
     .def("update_disp", &'''+name+'''::update_disp)
-    .def("generate_image", &'''+name+'''::generate_image)
+    .def("generate_image", &'''+name+'''::generate_image, pybind11::arg("n_down") = 1)
+    .def("compute_downsampled_image", &'''+name+'''::compute_downsampled_image, pybind11::arg("downsampling_factor_input"), pybind11::arg("write_temp_images") = 0, pybind11::arg("suffix") = "")
     .def("write_image", &'''+name+'''::write_image, pybind11::arg("filename"))'''+('''
     .def("write_grad_image", &'''+name+'''::write_grad_image, pybind11::arg("filename"))''')*(im_type=="grad")+'''
     .def("write_probe_image", &'''+name+'''::write_probe_image, pybind11::arg("filename"))
